package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special;

import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.json.ItemAdapter;
import dev.shadowsoffire.placebo.codec.IngredientCodec;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import net.minecraft.core.registries.Registries;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.storage.loot.parameters.LootContextParams;
import net.minecraft.world.level.storage.loot.LootContext;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import net.minecraft.ChatFormatting;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import net.minecraft.network.chat.Component;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.placebo.util.StepFunction;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.world.level.block.Block;
import net.minecraft.tags.TagKey;
import java.util.Optional;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;

public class DropTransformBonus extends GemBonus
{
    public static Codec<DropTransformBonus> CODEC;
    protected final Optional<TagKey<Block>> tag;
    protected final Ingredient inputs;
    protected final ItemStack output;
    protected final Map<LootRarity, StepFunction> values;
    protected final String descKey;
    
    public DropTransformBonus(final GemClass gemClass, final Optional<TagKey<Block>> tag, final Ingredient inputs, final ItemStack output, final Map<LootRarity, StepFunction> values, final String descKey) {
        super(Apotheosis.loc("drop_transform"), gemClass);
        this.tag = tag;
        this.inputs = inputs;
        this.output = output;
        this.values = values;
        this.descKey = descKey;
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return DropTransformBonus.CODEC;
    }
    
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final float chance = this.values.get(rarity).min();
        return (Component)Component.m_237110_(this.descKey, new Object[] { Affix.fmt(chance * 100.0f) }).m_130940_(ChatFormatting.YELLOW);
    }
    
    public void modifyLoot(final ItemStack gem, final LootRarity rarity, final ObjectArrayList<ItemStack> loot, final LootContext ctx) {
        if (ctx.m_78936_(LootContextParams.f_81461_)) {
            final BlockState state = (BlockState)ctx.m_165124_(LootContextParams.f_81461_);
            if (this.tag.isPresent() && !state.m_204336_((TagKey)this.tag.get())) {
                return;
            }
            if (ctx.m_230907_().m_188501_() <= this.values.get(rarity).min()) {
                for (int i = 0; i < loot.size(); ++i) {
                    final ItemStack stack = (ItemStack)loot.get(i);
                    if (this.inputs.test(stack)) {
                        final ItemStack outCopy = this.output.m_41777_();
                        outCopy.m_41764_(stack.m_41613_());
                        loot.set(i, (Object)outCopy);
                    }
                }
            }
        }
    }
    
    public DropTransformBonus validate() {
        Preconditions.checkNotNull(this.values);
        this.values.forEach((k, v) -> {
            Preconditions.checkNotNull(k);
            Preconditions.checkNotNull(v);
            return;
        });
        return this;
    }
    
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    public int getNumberOfUUIDs() {
        return 0;
    }
    
    static {
        DropTransformBonus.CODEC = (Codec<DropTransformBonus>)RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)PlaceboCodecs.nullableField(TagKey.m_203877_(Registries.f_256747_), "blocks").forGetter(a -> a.tag), (App)IngredientCodec.INSTANCE.fieldOf("inputs").forGetter(a -> a.inputs), (App)ItemAdapter.CODEC.fieldOf("output").forGetter(a -> a.output), (App)DropTransformBonus.VALUES_CODEC.fieldOf("values").forGetter(a -> a.values), (App)Codec.STRING.fieldOf("desc").forGetter(a -> a.descKey)).apply((Applicative)inst, DropTransformBonus::new));
    }
}
